"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _react = require("react");
var _lodash = _interopRequireDefault(require("lodash.throttle"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
/**
 * A custom hook that handles the state for the timer
 */
const useTimer = _ref => {
  let {
    initialTimeInMs = 0,
    onFinish = () => null,
    mode = 'stopwatch',
    intervalMs = 16
  } = _ref;
  const direction = mode === 'timer' ? -1 : 1;
  const [elapsedInMs, setElapsedInMs] = (0, _react.useState)(0);
  const startTime = (0, _react.useRef)(null);
  const pausedTime = (0, _react.useRef)(null);
  const intervalId = (0, _react.useRef)(null);
  const throttledOnFinish = (0, _react.useMemo)(() => (0, _lodash.default)(onFinish, 100, {
    trailing: false
  }),
  // eslint-disable-next-line react-hooks/exhaustive-deps
  []);
  (0, _react.useEffect)(() => {
    // Ensure that the timer is reset when the initialTimeInMs changes
    reset();
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [initialTimeInMs]);
  (0, _react.useEffect)(() => {
    // Checking if it's a timer and it reached 0
    if (mode === 'timer' && elapsedInMs >= initialTimeInMs) {
      removeInterval();
      setElapsedInMs(initialTimeInMs);
      throttledOnFinish();
    }
  }, [elapsedInMs, initialTimeInMs, mode, throttledOnFinish]);
  function getSnapshot() {
    return Math.abs(initialTimeInMs + elapsedInMs * direction);
  }
  function play() {
    // Already playing, returning early
    if (intervalId.current) {
      return;
    }
    // Timer mode and it reached 0, returning early
    if (elapsedInMs === initialTimeInMs && mode === 'timer') {
      return;
    }
    // First time playing, recording the start time
    if (!startTime.current) {
      startTime.current = Date.now();
    }
    intervalId.current = setInterval(() => {
      if (!pausedTime.current) {
        setElapsedInMs(Date.now() - startTime.current);
      } else {
        // If the timer is paused, we need to update the start time
        const elapsedSincePaused = Date.now() - pausedTime.current;
        startTime.current = startTime.current + elapsedSincePaused;
        pausedTime.current = null;
      }
    }, intervalMs);
  }
  function resetState() {
    setElapsedInMs(0);
    startTime.current = null;
    pausedTime.current = null;
  }
  function removeInterval() {
    if (intervalId.current) {
      clearInterval(intervalId.current);
      intervalId.current = null;
    }
  }
  function pause() {
    removeInterval();
    if (!pausedTime.current && elapsedInMs > 0) {
      pausedTime.current = Date.now();
    }
    return getSnapshot();
  }
  function reset() {
    removeInterval();
    resetState();
  }
  const countInSeconds = Math.floor(getSnapshot() / 1000);
  return {
    tensOfMs: Math.floor(getSnapshot() / 10) % 100,
    lastDigit: countInSeconds % 10,
    tens: Math.floor(countInSeconds / 10) % 6,
    minutes: Math.floor(countInSeconds / 60),
    play,
    pause,
    reset,
    getSnapshot
  };
};
var _default = useTimer;
exports.default = _default;
//# sourceMappingURL=useTimer.js.map